//
//
/*
    Copyright (C) 2016, UCL
    This file is part of STIR.

    Most of this file is free software; you can redistribute that part and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This file is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    See STIR/LICENSE.txt for details
*/
/*!
  \file
  \ingroup scatter
  \brief File that registers all stir::RegisterObject children in scatter estimation

  \author Nikos Efthimiou

*/
#include "stir/scatter/SingleScatterSimulation.h"

START_NAMESPACE_STIR

static SingleScatterSimulation::RegisterIt dummy699;

END_NAMESPACE_STIR
